import React, { createContext, useMemo, useContext, useCallback } from 'react';
import useSWR, { createCache, SWRConfig } from 'swr';

const RARIBLE_MATCH_RE = /^https:\/\/rarible\.com\/token\/(0x[a-fA-F0-9]{40}):([0-9]+)/;
function isAddress(value) {
    return /^0x[a-fA-F0-9]{40}$/.test(value);
}
function parseNftUrl(url) {
    const raribleMatch = RARIBLE_MATCH_RE.exec(url);
    if (raribleMatch) {
        return [raribleMatch[1], raribleMatch[2]];
    }
    return null;
}
function fetchImage(src) {
    return new Promise((resolve, reject) => {
        const image = new Image();
        image.src = src;
        image.crossOrigin = "";
        image.onload = () => resolve(image);
        image.onerror = (error) => reject(error);
    });
}
// Scale the image and add some extra padding. Returns the image as base64.
// The padding and scale are expressed as proportions of the image size.
function frameImage(image, { scale = 1, padding = 0 } = {}) {
    const width = image.naturalWidth * scale;
    const height = image.naturalHeight * scale;
    const _padding = Math.max(width * padding, height * padding);
    const canvas = document.createElement("canvas");
    canvas.width = width + _padding * 2;
    canvas.height = height + _padding * 2;
    const ctx = canvas.getContext("2d");
    if (ctx === null) {
        return null;
    }
    ctx.imageSmoothingEnabled = false;
    ctx.drawImage(image, _padding, _padding, width, height);
    return canvas.toDataURL();
}
function ipfsUrlDefault(cid, path = "") {
    return `https://ipfs.io/ipfs/${cid}${path}`;
}
const IPFS_PROTOCOL_RE = /^ipfs:\/\/(?:ipfs\/)?([^/]+)(\/.+)?$/;
const IPFS_HASH_RE = /^Qm[1-9A-HJ-NP-Za-km-z]{44}$/;
function ipfsUrl(url, ipfsUrl = ipfsUrlDefault) {
    const ipfsProtocolMatch = IPFS_PROTOCOL_RE.exec(url);
    if (ipfsProtocolMatch) {
        const [, cid, path = ""] = ipfsProtocolMatch;
        return ipfsUrl(cid, path);
    }
    // not perfect, but should be enough
    if (IPFS_HASH_RE.test(url)) {
        return ipfsUrl(url);
    }
    return url;
}
function normalizeOpenSeaUrl(url, tokenId) {
    // url can be anything so we need to try / catch to pass it to new URL()
    try {
        const _url = new URL(url);
        // 0x%7Bid%7D" = 0x{id} (url encoded)
        if ((_url.host !== "api.opensea.io" &&
            _url.host !== "testnets-api.opensea.io") ||
            !_url.pathname.includes("0x%7Bid%7D")) {
            return url;
        }
        _url.pathname = _url.pathname.replace(/0x%7Bid%7D/g, tokenId);
        _url.searchParams.set("format", "json");
        return String(_url);
    }
    catch (err) {
        return url;
    }
}
function normalizeNiftyGatewayUrl(url) {
    try {
        const _url = new URL(url);
        if (_url.host !== "api.niftygateway.com") {
            return url;
        }
        // Without final slash, the Nifty Gateway API server
        // doesn’t set the CORS headers properly.
        _url.pathname = _url.pathname + "/";
        return String(_url);
    }
    catch (err) {
        return url;
    }
}
function normalizeTokenUrl(url, tokenId) {
    url = normalizeOpenSeaUrl(url, tokenId);
    url = normalizeNiftyGatewayUrl(url);
    url = ipfsUrl(url);
    return url;
}
function normalizeImageUrl(url) {
    return ipfsUrl(url);
}
function normalizeNftMetadata(data) {
    return {
        ...data,
        image: normalizeImageUrl(data.image),
    };
}
function fixIncorrectImageField(data) {
    if (!data || typeof data !== "object") {
        return data;
    }
    const _data = data;
    // makersplace.com is using `imageUrl` rather than `image`
    if (typeof (_data === null || _data === void 0 ? void 0 : _data.image) === "undefined" &&
        typeof (_data === null || _data === void 0 ? void 0 : _data.imageUrl) === "string") {
        return { ..._data, image: _data === null || _data === void 0 ? void 0 : _data.imageUrl };
    }
    return data;
}
// See NftMetadataMixedInJsonSchema for why this is needed.
function isNftMetadataMixedInJsonSchema(data) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    if (!data || typeof data !== "object") {
        return false;
    }
    const _data = data;
    return (_data.title === "Asset Metadata" &&
        _data.type === "object" &&
        typeof ((_b = (_a = _data.properties) === null || _a === void 0 ? void 0 : _a.name) === null || _b === void 0 ? void 0 : _b.description) === "string" &&
        typeof ((_d = (_c = _data.properties) === null || _c === void 0 ? void 0 : _c.image) === null || _d === void 0 ? void 0 : _d.description) === "string" &&
        typeof ((_f = (_e = _data.properties) === null || _e === void 0 ? void 0 : _e.description) === null || _f === void 0 ? void 0 : _f.description) === "string" &&
        ((_h = (_g = _data.properties) === null || _g === void 0 ? void 0 : _g.name) === null || _h === void 0 ? void 0 : _h.type) === "string" &&
        ((_k = (_j = _data.properties) === null || _j === void 0 ? void 0 : _j.image) === null || _k === void 0 ? void 0 : _k.type) === "string" &&
        ((_m = (_l = _data.properties) === null || _l === void 0 ? void 0 : _l.description) === null || _m === void 0 ? void 0 : _m.type) === "string");
}
function fixNftMetadataMixedInJsonSchema(data) {
    var _a, _b, _c, _d, _e, _f;
    return {
        name: ((_b = (_a = data === null || data === void 0 ? void 0 : data.properties) === null || _a === void 0 ? void 0 : _a.name) === null || _b === void 0 ? void 0 : _b.description) || "",
        description: ((_d = (_c = data === null || data === void 0 ? void 0 : data.properties) === null || _c === void 0 ? void 0 : _c.description) === null || _d === void 0 ? void 0 : _d.description) || "",
        image: ((_f = (_e = data === null || data === void 0 ? void 0 : data.properties) === null || _e === void 0 ? void 0 : _e.image) === null || _f === void 0 ? void 0 : _f.description) || "",
    };
}
function isNftMetadata(data) {
    if (!data || typeof data !== "object") {
        return false;
    }
    const _data = data;
    // We don’t test for the exact type here, because some
    // NFT minting services set some of the fields as null.
    return "name" in _data && "image" in _data && "description" in _data;
}
function addressesEqual(addr1, addr2) {
    return (addr1 === null || addr1 === void 0 ? void 0 : addr1.toLowerCase()) === (addr2 === null || addr2 === void 0 ? void 0 : addr2.toLowerCase());
}
// Promise.any() implementation from https://github.com/m0ppers/promise-any
function promiseAny(promises) {
    return reversePromise(Promise.all([...promises].map(reversePromise)));
}
function reversePromise(promise) {
    return new Promise((resolve, reject) => {
        Promise.resolve(promise).then(reject, resolve);
    });
}

async function fetchMetadata(url) {
    const res = await fetch(url);
    if (!res.ok) {
        throw new Error("Error when trying to request " + url);
    }
    let data;
    try {
        data = await res.json();
    }
    catch (err) {
        // If it can’t be parsed as JSON, it must be an image URL
        data = { name: "", description: "", image: url };
    }
    if (isNftMetadataMixedInJsonSchema(data)) {
        data = fixNftMetadataMixedInJsonSchema(data);
    }
    data = fixIncorrectImageField(data);
    if (!isNftMetadata(data)) {
        throw new Error("Invalid data received");
    }
    return normalizeNftMetadata({
        name: data.name || "",
        image: data.image || "",
        description: data.description || "",
    });
}

const CRYPTOKITTIES = "0x06012c8cf97BEaD5deAe237070F9587f8E7A266d";
const CRYPTOPUNKS = "0xb47e3cd837ddf8e4c57f05d70ab865de6e193bbb";
const CRYPTOVOXELS = "0x79986aF15539de2db9A5086382daEdA917A9CF0C";
const DECENTRALAND_ESTATE = "0x959e104E1a4dB6317fA58F8295F586e1A978c297";
const DECENTRALAND_PARCEL = "0xf87e31492faf9a91b02ee0deaad50d51d56d5d4d";

const CRYPTOPUNKS_DESCRIPTION = `
  10,000 unique collectible characters with proof of ownership stored on the
  Ethereum blockchain. The project that inspired the modern CryptoArt movement.
  The first "Non-Fungible Token," and inspiration for the Ethereum ERC-721
  standard that powers most digital art and collectibles.
`;
function cryptoPunksMetadata(index) {
    return {
        description: CRYPTOPUNKS_DESCRIPTION,
        image: `https://www.larvalabs.com/cryptopunks/cryptopunk${index}.png`,
        name: `CryptoPunk ${index}`,
        owner: "",
    };
}
function isCryptoPunks(contractAddress) {
    return addressesEqual(contractAddress, CRYPTOPUNKS);
}

async function cryptoKittiesMetadata(id) {
    var _a, _b, _c;
    const res = await fetch(`https://api.cryptokitties.co/v3/kitties/${id}`);
    const data = (await res.json());
    return {
        description: (_a = data === null || data === void 0 ? void 0 : data.bio) !== null && _a !== void 0 ? _a : "−",
        image: (_b = data === null || data === void 0 ? void 0 : data.image_url) !== null && _b !== void 0 ? _b : "",
        name: (_c = data === null || data === void 0 ? void 0 : data.name) !== null && _c !== void 0 ? _c : "Unknown",
        owner: "",
    };
}
function isCryptoKitties(contractAddress) {
    return addressesEqual(contractAddress, CRYPTOKITTIES);
}

const ENDPOINT$1 = "https://api.thegraph.com/subgraphs/name/decentraland/marketplace";
const QUERY$1 = `
  query NFTByTokenId($contractAddress: String, $tokenId: String) {
    nfts(
      where: { contractAddress: $contractAddress, tokenId: $tokenId }
      first: 1
    ) {
      name
      image
      owner {
        address
      }
      estate {
        size
        data {
          description
        }
      }
    }
  }
`;
function body$1(contractAddress, tokenId) {
    return JSON.stringify({
        operationName: "NFTByTokenId",
        variables: { contractAddress, tokenId },
        query: QUERY$1,
    });
}
async function decentralandEstateMetadata(tokenId) {
    var _a, _b, _c, _d, _e, _f, _g, _h;
    const response = await fetch(ENDPOINT$1, {
        body: body$1(DECENTRALAND_ESTATE, tokenId),
        method: "POST",
    });
    const { data } = (await response.json());
    const nft = (_a = data === null || data === void 0 ? void 0 : data.nfts) === null || _a === void 0 ? void 0 : _a[0];
    return {
        description: (_d = (_c = (_b = nft === null || nft === void 0 ? void 0 : nft.estate) === null || _b === void 0 ? void 0 : _b.data) === null || _c === void 0 ? void 0 : _c.description) !== null && _d !== void 0 ? _d : "−",
        image: (_e = nft === null || nft === void 0 ? void 0 : nft.image) !== null && _e !== void 0 ? _e : "",
        name: (_f = nft === null || nft === void 0 ? void 0 : nft.name) !== null && _f !== void 0 ? _f : "Unknown",
        owner: (_h = (_g = nft === null || nft === void 0 ? void 0 : nft.owner) === null || _g === void 0 ? void 0 : _g.address) !== null && _h !== void 0 ? _h : "",
    };
}
function isDecentralandEstate(contractAddress) {
    return addressesEqual(contractAddress, DECENTRALAND_ESTATE);
}

const ENDPOINT = "https://api.thegraph.com/subgraphs/name/decentraland/marketplace";
const QUERY = `
  query NFTByTokenId($contractAddress: String, $tokenId: String) {
    nfts(
      where: { contractAddress: $contractAddress, tokenId: $tokenId }
      first: 1
    ) {
      name
      image
      owner {
        address
      }
      parcel {
        x
        y
        data {
          description
        }
      }
    }
  }
`;
function body(contractAddress, tokenId) {
    return JSON.stringify({
        operationName: "NFTByTokenId",
        variables: { contractAddress, tokenId },
        query: QUERY,
    });
}
async function decentralandParcelMetadata(tokenId) {
    var _a, _b, _c, _d, _e, _f, _g;
    const response = await fetch(ENDPOINT, {
        body: body(DECENTRALAND_PARCEL, tokenId),
        method: "POST",
    });
    const { data } = (await response.json());
    const nft = (_a = data === null || data === void 0 ? void 0 : data.nfts) === null || _a === void 0 ? void 0 : _a[0];
    const parcel = nft === null || nft === void 0 ? void 0 : nft.parcel;
    return {
        description: (_c = (_b = parcel === null || parcel === void 0 ? void 0 : parcel.data) === null || _b === void 0 ? void 0 : _b.description) !== null && _c !== void 0 ? _c : "−",
        image: (_d = nft === null || nft === void 0 ? void 0 : nft.image) !== null && _d !== void 0 ? _d : "",
        name: (_e = nft === null || nft === void 0 ? void 0 : nft.name) !== null && _e !== void 0 ? _e : `Parcel ${parcel === null || parcel === void 0 ? void 0 : parcel.x},${parcel === null || parcel === void 0 ? void 0 : parcel.y}`,
        owner: (_g = (_f = nft === null || nft === void 0 ? void 0 : nft.owner) === null || _f === void 0 ? void 0 : _f.address) !== null && _g !== void 0 ? _g : "",
    };
}
function isDecentralandParcel(contractAddress) {
    return addressesEqual(contractAddress, DECENTRALAND_PARCEL);
}

const MOONCATS_WRAPPED = {
    address: "0x7c40c393dc0f283f318791d746d894ddd3693572",
    methodName: "_tokenIDToCatID",
    methodHash: "0xfe294644",
    humanReadableAbi: [
        "function _tokenIDToCatID(uint256 tokenId) view returns (bytes5 catId)",
    ],
};
// See https://www.reddit.com/r/MoonCatRescue/comments/m5d7mx/svg_imagery_of_all_rescued_mooncats/
const MOONCATS_IPFS_CID = "bafybeidk4zunuq56w2pf2sncexohlyqae62dzplljkbwswa7jwywh2dava";
async function imageUrl(catId) {
    const dir = catId.slice(4, 6);
    const url = ipfsUrl(`ipfs://ipfs/${MOONCATS_IPFS_CID}/${dir}/${catId}.png`);
    const image = await fetchImage(url);
    // Here we increase the resolution of the MoonCats PNG files (4x without
    // antialias) and add a some padding around it. Doing this image manipulation
    // rather than using CSS is needed because useNft() only deals with data and
    // doesn’t have any control over how the images get displayed.
    return frameImage(image, { scale: 4, padding: 0.125 });
}
async function moonCatsMetadata(tokenId, getCatId) {
    var _a;
    const catId = await getCatId(MOONCATS_WRAPPED.address, tokenId, MOONCATS_WRAPPED);
    const image = (_a = (await imageUrl(catId))) !== null && _a !== void 0 ? _a : "";
    return {
        name: `Wrapped MoonCat #${tokenId}`,
        description: `The (unofficial) wrapped version of MoonCats Rescue. ` +
            `Original cat ID: ${catId}.`,
        image,
        owner: "",
    };
}
function isMoonCats(contractAddress) {
    return addressesEqual(contractAddress, MOONCATS_WRAPPED.address);
}

function moonCatsCatId$1(config) {
    return async function moonCatsCatId(contractAddress, tokenId, method) {
        const wrappedContract = new config.ethers.Contract(contractAddress, method.humanReadableAbi, config.provider);
        const result = await wrappedContract._tokenIDToCatID(tokenId);
        return result !== null && result !== void 0 ? result : "";
    };
}

const ABI = [
    // ERC-721
    "function tokenURI(uint256 _tokenId) external view returns (string)",
    "function ownerOf(uint256 _tokenId) external view returns (address)",
    // ERC-1155
    "function uri(uint256 _id) external view returns (string)",
    // ERC-165
    "function supportsInterface(bytes4 interfaceID) external view returns (bool)",
];
async function url(contract, tokenId) {
    const uri = await promiseAny([
        contract.uri(tokenId),
        contract.tokenURI(tokenId),
    ]);
    return normalizeTokenUrl(uri, tokenId);
}
async function fetchStandardNftContractData$1(contractAddress, tokenId, config) {
    const contract = new config.ethers.Contract(contractAddress, ABI, config.provider);
    return Promise.all([
        url(contract, tokenId),
        contract.ownerOf(tokenId).catch(() => ""),
    ]);
}

function ethersFetcher(config) {
    return {
        config,
        async fetchNft(contractAddress, tokenId) {
            if (!isAddress(contractAddress)) {
                throw new Error(`Invalid contract address: ${contractAddress}`);
            }
            if (isDecentralandParcel(contractAddress)) {
                return decentralandParcelMetadata(tokenId);
            }
            if (isDecentralandEstate(contractAddress)) {
                return decentralandEstateMetadata(tokenId);
            }
            if (isCryptoPunks(contractAddress)) {
                return cryptoPunksMetadata(tokenId);
            }
            if (isCryptoKitties(contractAddress)) {
                return cryptoKittiesMetadata(tokenId);
            }
            if (isMoonCats(contractAddress)) {
                return moonCatsMetadata(tokenId, moonCatsCatId$1(config));
            }
            const [metadataUrl, owner] = await fetchStandardNftContractData$1(contractAddress, tokenId, config);
            const metadata = await fetchMetadata(metadataUrl);
            return { ...metadata, owner };
        },
    };
}

// Pre encoded method names to avoid embedding a keccak256 library.
// See https://docs.soliditylang.org/en/v0.5.3/abi-spec.html#function-selector-and-argument-encoding
const URI_METHOD_ERC721 = "0xc87b56dd"; // tokenURI(uint256)
const URI_METHOD_ERC1155 = "0x0e89341c"; // uri(uint256)
const OWNER_OF_METHOD_ERC721 = "0x6352211e"; // ownerOf(uint256)
const SUPPORTS_INTERFACE_METHOD_ERC165 = "0x01ffc9a7"; // supportsInterface(bytes4)
// ERC165 identifiers
const ERC721_ID = "0x80ac58cd";
const ERC1155_ID = "0xd9b67a26";
// Utilities adapted from https://github.com/Zoltu/ethereum-abi-encoder/
function uint256Hex(value) {
    let result = "";
    for (let i = 0; i < 32; ++i) {
        result += ("0" + ((value >> BigInt(8 * 32 - i * 8 - 8)) & BigInt(255)).toString(16)).slice(-2);
    }
    return result;
}
function bytesToBigInt(bytes) {
    let value = BigInt(0);
    for (const byte of bytes) {
        value = (value << BigInt(8)) + BigInt(byte);
    }
    return value;
}
function hexToUint8Array(hex) {
    var _a;
    hex = hex.replace(/^0x/, "");
    return new Uint8Array(((_a = hex.match(/.{1,2}/g)) !== null && _a !== void 0 ? _a : []).map((byte) => parseInt(byte, 16)));
}
function decodeBoolean(hex) {
    return bytesToBigInt(hexToUint8Array(hex)) !== BigInt(0);
}
function decodeString(hex) {
    const data = hexToUint8Array(hex);
    const pointer = Number(bytesToBigInt(data.subarray(0, 32)));
    const length = Number(bytesToBigInt(data.subarray(pointer, pointer + 32)));
    const bytes = data.subarray(pointer + 32, pointer + 32 + length);
    return new TextDecoder().decode(bytes);
}
function decodeAddress(hex) {
    const data = hexToUint8Array(hex);
    const bytes = data.subarray(0, 32);
    const decoded = bytesToBigInt(bytes);
    if (decoded >= BigInt(2) ** BigInt(160))
        throw new Error(`Encoded value is bigger than the largest possible address.  Decoded value: 0x${decoded.toString(16)}.`);
    return `0x${decoded.toString(16)}`;
}
function methodUriErc721(tokenId) {
    return URI_METHOD_ERC721 + uint256Hex(tokenId);
}
function methodUriErc1155(id) {
    return URI_METHOD_ERC1155 + uint256Hex(id);
}
function methodOwnerOfErc721(tokenId) {
    return OWNER_OF_METHOD_ERC721 + uint256Hex(tokenId);
}
function supportsInterfaceMethodErc165(interfaceId) {
    return (SUPPORTS_INTERFACE_METHOD_ERC165 +
        interfaceId.replace(/^0x/, "").padEnd(64, "0"));
}
function ethCall(ethereum, to, data) {
    return ethereum.request({
        method: "eth_call",
        params: [{ data, to }, "latest"],
    });
}

function moonCatsCatId(config) {
    return async function moonCatsCatId(contractAddress, tokenId, method) {
        if (config.ethereum === undefined) {
            throw new Error("No Ethereum provider");
        }
        const result = await ethCall(config.ethereum, contractAddress, method.methodHash + uint256Hex(BigInt(tokenId)));
        return result.slice(0, 12); // 12 = 0x prefix + 5 bytes
    };
}

// Contracts implementing ERC721’s tokenURI()
// but are missing ERC165’s supportsInterface()
const KNOWN_ERC721_LIKE = [CRYPTOVOXELS];
async function fetchStandardNftContractData(contractAddress, tokenId, { ethereum }) {
    if (!ethereum) {
        return ["", ""];
    }
    const urlCall = async (method) => {
        const result = await ethCall(ethereum, contractAddress, method);
        return normalizeTokenUrl(decodeString(result), tokenId);
    };
    // call ownerOf() even on 1155 contracts, just in case it exists
    const ownerPromise = ethCall(ethereum, contractAddress, methodOwnerOfErc721(BigInt(tokenId)))
        .then(decodeAddress)
        .catch(() => "");
    const isKnown721Like = KNOWN_ERC721_LIKE.some((address) => addressesEqual(address, contractAddress));
    if (isKnown721Like) {
        return Promise.all([
            urlCall(methodUriErc721(BigInt(tokenId))),
            ownerPromise,
        ]);
    }
    const calls = [
        [
            supportsInterfaceMethodErc165(ERC721_ID),
            methodUriErc721(BigInt(tokenId)),
        ],
        [
            supportsInterfaceMethodErc165(ERC1155_ID),
            methodUriErc1155(BigInt(tokenId)),
        ],
    ];
    try {
        return promiseAny(calls.map(async ([supportsMethod, uriMethod]) => {
            // Check if the interface is supported first
            const supported = await ethCall(ethereum, contractAddress, supportsMethod).then(decodeBoolean);
            // throw for the Promise.any() to skip this branch
            if (!supported) {
                throw new Error("Unsupported method");
            }
            return Promise.all([urlCall(uriMethod), ownerPromise]);
        }));
    }
    catch (err) {
        return ["", ""];
    }
}

function ethereumFetcher(config) {
    return {
        config,
        async fetchNft(contractAddress, tokenId) {
            if (!isAddress(contractAddress)) {
                throw new Error(`Invalid contract address: ${contractAddress}`);
            }
            if (isDecentralandParcel(contractAddress)) {
                return decentralandParcelMetadata(tokenId);
            }
            if (isDecentralandEstate(contractAddress)) {
                return decentralandEstateMetadata(tokenId);
            }
            if (isCryptoPunks(contractAddress)) {
                return cryptoPunksMetadata(tokenId);
            }
            if (isCryptoKitties(contractAddress)) {
                return cryptoKittiesMetadata(tokenId);
            }
            if (isMoonCats(contractAddress)) {
                return moonCatsMetadata(tokenId, moonCatsCatId(config));
            }
            const [metadataUrl, owner] = await fetchStandardNftContractData(contractAddress, tokenId, config);
            const metadata = await fetchMetadata(metadataUrl);
            return { ...metadata, owner };
        },
    };
}

const NFT_METADATA_DEFAULT$1 = {
    name: "",
    description: "",
    image: "",
};
function isFetcherDeclarationEthers(fetcher) {
    return (Array.isArray(fetcher) && fetcher.length == 2 && fetcher[0] === "ethers");
}
function isFetcherDeclarationEthereum(fetcher) {
    return (Array.isArray(fetcher) && fetcher.length == 2 && fetcher[0] === "ethereum");
}
function normalizeFetcher(fetcher) {
    // default fetcher
    if (!fetcher) {
        return {
            config: {},
            fetchNft: () => Promise.resolve(NFT_METADATA_DEFAULT$1),
        };
    }
    // ethers
    if (isFetcherDeclarationEthers(fetcher)) {
        return ethersFetcher(fetcher[1]);
    }
    // ethereum
    if (isFetcherDeclarationEthereum(fetcher)) {
        return ethereumFetcher(fetcher[1]);
    }
    // custom fetcher (or wrong value)
    return fetcher;
}
const NftContext = createContext(null);
const NftProvider = function NftProvider({ children, fetcher }) {
    const [cacheStorage, { cache: swrCache }] = useMemo(() => {
        const cache = new Map();
        return [cache, createCache(cache)];
    }, []);
    const contextValue = {
        cacheStorage,
        fetcher: normalizeFetcher(fetcher),
    };
    return (React.createElement(SWRConfig, { value: { cache: swrCache } },
        React.createElement(NftContext.Provider, { value: contextValue }, children)));
};
function useNft(contractAddress, tokenId) {
    const context = useContext(NftContext);
    if (context === null) {
        throw new Error("Please wrap your app with <NftProvider />");
    }
    const { fetcher, cacheStorage } = context;
    const fetchNft = useCallback(() => {
        return fetcher
            ? fetcher.fetchNft(contractAddress, tokenId)
            : { ...NFT_METADATA_DEFAULT$1 };
    }, [contractAddress, fetcher, tokenId]);
    const cached = cacheStorage.has(contractAddress + tokenId);
    const result = useSWR(contractAddress + tokenId, fetchNft, {
        revalidateOnMount: !cached,
        revalidateOnFocus: false,
        revalidateOnReconnect: false,
    });
    return useMemo(() => {
        const { error, data, revalidate } = result;
        const reload = () => revalidate();
        if (error === undefined && data === undefined) {
            return {
                error: undefined,
                loading: true,
                nft: undefined,
                reload,
                status: "loading",
            };
        }
        if (error !== undefined) {
            return {
                error,
                loading: false,
                nft: undefined,
                reload,
                status: "error",
            };
        }
        return {
            error: undefined,
            loading: false,
            nft: data,
            reload,
            status: "done",
        };
    }, [result]);
}

const NFT_METADATA_DEFAULT = {
    name: "",
    description: "",
    image: "",
};
class FetchWrapper {
    constructor(fetcher) {
        this.fetcher = this.normalizeFetcher(fetcher);
    }
    normalizeFetcher(fetcher) {
        // default fetcher
        if (!fetcher) {
            return {
                config: {},
                fetchNft: () => Promise.resolve(NFT_METADATA_DEFAULT),
            };
        }
        // ethers
        if (this.isFetcherDeclarationEthers(fetcher)) {
            return ethersFetcher(fetcher[1]);
        }
        // ethereum
        if (this.isFetcherDeclarationEthereum(fetcher)) {
            return ethereumFetcher(fetcher[1]);
        }
        // custom fetcher (or wrong value)
        return fetcher;
    }
    isFetcherDeclarationEthers(fetcher) {
        return (Array.isArray(fetcher) && fetcher.length == 2 && fetcher[0] === "ethers");
    }
    isFetcherDeclarationEthereum(fetcher) {
        return (Array.isArray(fetcher) && fetcher.length == 2 && fetcher[0] === "ethereum");
    }
    async fetchNft(contractAddress, tokenId) {
        return await this.fetcher.fetchNft(contractAddress, tokenId);
    }
}

export { FetchWrapper, NftProvider, ethereumFetcher, ethersFetcher, parseNftUrl, useNft };
//# sourceMappingURL=use-nft.esm.js.map
